@@ -17,7 +17,15 @@ use vulkano::{
     buffer::{BufferUsage, CpuAccessibleBuffer, TypedBufferAccess},
     command_buffer::{AutoCommandBufferBuilder, CommandBufferUsage, SecondaryAutoCommandBuffer},
     device::Queue,
-    pipeline::{viewport::Viewport, GraphicsPipeline},
+    pipeline::{
+        graphics::{
+            depth_stencil::DepthStencilState,
+            input_assembly::InputAssemblyState,
+            vertex_input::BuffersDefinition,
+            viewport::{Viewport, ViewportState},
+        },
+        GraphicsPipeline,
+    },
     render_pass::Subpass,
 };
 
@@ -45,23 +53,19 @@ impl TriangleDrawSystem {
             .expect("failed to create buffer")
         };
         let pipeline = {
-            let vs = vs::Shader::load(gfx_queue.device().clone())
-                .expect("failed to create shader module");
-            let fs = fs::Shader::load(gfx_queue.device().clone())
-                .expect("failed to create shader module");
+            let vs = vs::load(gfx_queue.device().clone()).expect("failed to create shader module");
+            let fs = fs::load(gfx_queue.device().clone()).expect("failed to create shader module");
 
-            Arc::new(
-                GraphicsPipeline::start()
-                    .vertex_input_single_buffer::<Vertex>()
-                    .vertex_shader(vs.main_entry_point(), ())
-                    .triangle_list()
-                    .viewports_dynamic_scissors_irrelevant(1)
-                    .fragment_shader(fs.main_entry_point(), ())
-                    .depth_stencil_simple_depth()
-                    .render_pass(subpass)
-                    .build(gfx_queue.device().clone())
-                    .unwrap(),
-            ) as Arc<_>
+            GraphicsPipeline::start()
+                .vertex_input_state(BuffersDefinition::new().vertex::<Vertex>())
+                .vertex_shader(vs.entry_point("main").unwrap(), ())
+                .input_assembly_state(InputAssemblyState::new())
+                .fragment_shader(fs.entry_point("main").unwrap(), ())
+                .viewport_state(ViewportState::viewport_dynamic_scissor_irrelevant())
+                .depth_stencil_state(DepthStencilState::simple_depth_test())
+                .render_pass(subpass)
+                .build(gfx_queue.device().clone())
+                .unwrap()
         };
 
         TriangleDrawSystem { gfx_queue, vertex_buffer, pipeline }
